"""
Programmatic API for running the WorkflowEngine.
Provides a simplified interface for use in Python scripts.
"""

import os
from typing import Optional, Dict, Any, Union

from dotenv import load_dotenv

from core.workflow_engine import WorkflowEngine, WorkflowEngineConfig


def run_workflow(
    issue_id: int,
    requirement: str,
    project_dir: Optional[str] = None,
    model: Optional[str] = None,  # 统一的模型设置
    core_model: Optional[str] = "gpt-4o",
    data_model: Optional[str] = None,  # 默认与core_model相同
    temperature: Optional[float] = None,  # 统一的温度设置
    core_temperature: float = 0.7,
    data_temperature: float = 0.7,
    max_retry: int = 3,
    default_branch: str = "main",
    mode: str = "client",
    base_url: Optional[str] = None,
    api_key: Optional[str] = None,
    github_remote_url: Optional[str] = None,
    github_token: Optional[str] = None,
    **kwargs: Dict[str, Any]
) -> None:
    """Run the WorkflowEngine with the given configuration."""
    # Load environment variables
    load_dotenv()
    
    # Use current directory if no project_dir specified
    if project_dir is None:
        project_dir = os.getcwd()
    
    # 处理统一的模型配置
    if model is not None:
        core_model = model
        data_model = model
    
    # 如果未指定data_model，则默认与core_model相同
    if data_model is None:
        data_model = core_model
    
    # 处理统一的温度配置
    if temperature is not None:
        core_temperature = temperature
        data_temperature = temperature
    
    # Create config with provided parameters
    config = WorkflowEngineConfig(
        project_dir=project_dir, issue_id=issue_id, 
        core_model=core_model, data_model=data_model,
        core_template=core_temperature, data_template=data_temperature,
        max_retry=max_retry, default_branch=default_branch, mode=mode, 
        base_url=base_url, api_key=api_key, github_remote_url=github_remote_url,
        github_token=github_token
    )
    
    # Run the workflow engine
    engine = WorkflowEngine(config)
    response = engine.process_requirement(requirement)
    
    return response



"""
Terminal entrypoint for the WorkflowEngine.
Provides functionality to run the engine from terminal with command-line arguments.
"""
import os
import sys

from dotenv import load_dotenv

from client.cli import parse_args, get_requirement_text, build_config_from_args
from core.log_config import setup_logging
from core.workflow_engine import WorkflowEngine, WorkflowEngineConfig


def run_workflow_from_terminal() -> str:
    """
    Main entry point for running the workflow engine from terminal.
    Parses command line arguments and runs the workflow engine.
    """
    # Load environment variables from .env file if present
    load_dotenv()
    
    # Parse command line arguments
    args = parse_args()
    
    # Get requirement text
    requirement = get_requirement_text(args)
    if not requirement:
        sys.exit(1)
    
    # Build config from arguments
    config_params = build_config_from_args(args)
    
    # Try to get API key from environment if not provided as argument
    if "api_key" not in config_params and os.environ.get("OPENAI_API_KEY"):
        config_params["api_key"] = os.environ.get("OPENAI_API_KEY")
    
    # Create the workflow engine config
    config = WorkflowEngineConfig(**config_params)
    
    # Initialize and run the workflow engine
    engine = WorkflowEngine(config)
    response = engine.process_requirement(requirement)
    
    # Print the response to the terminal if available
    if response:
        print(f"\nResponse:\n{response}")
    
    return response if response else ""


if __name__ == "__main__":
    setup_logging()
    response = run_workflow_from_terminal()




import os
import re
from typing import List, Tuple

from dotenv import load_dotenv
from langchain_core.tools import StructuredTool
from pydantic import BaseModel, Field

from core.ai import AIAssistant, AIConfig
from core.log_config import get_logger

logger = get_logger(__name__)

class DiffInfo(BaseModel):
    """存储diff信息的数据类"""
    file_name: str = ""  # 文件路径
    content: str = ""    # diff内容或新文件内容
    file_content: str = ""  # 文件的原始内容（如果是修改文件）
    is_create: bool = False  # 是否为新建文件
    is_modify: bool = False  # 是否为修改文件
    is_delete: bool = False  # 是否为删除文件


class Diff:
    """
    处理Git diff格式的工具类
    使用 AI 模型生成新文件内容
    """
    
    def __init__(self, ai_config: AIConfig):
        """
        初始化 Diff 类
        
        Args:
            ai_config: AI 配置
        """
        # 保存 AI 配置
        self.ai_config = ai_config
        
        # 保存原始系统提示词
        self.original_sys_prompt = ai_config.sys_prompt
        
        # 设置处理 diff 的系统提示词
        ai_config.sys_prompt = self._get_diff_system_prompt()
        
        # 初始化工具列表
        self.tools = [self._create_replace_file_tool()]
        
        # 创建 AI 助手
        self.ai_assistant = AIAssistant(config=ai_config, tools=self.tools)
    
    def __del__(self):
        """析构函数，恢复原始系统提示词"""
        self.ai_config.sys_prompt = self.original_sys_prompt

    @staticmethod
    def parse_diffs_from_text(text: str) -> List[Tuple[str, str, str]]:
        """
        从文本中提取所有文件修改信息

        Args:
            text: 包含文件修改信息的文本

        Returns:
            List[Tuple[str, str, str]]: 解析后的文件信息列表，每个元素为 (原文件路径, 新文件路径, diff内容)
        """
        diffs = []
        last_file_path = None  # 记录前一个文件的路径
        
        logger.info(f"开始解析文本中的 diff 信息，文本长度: {len(text)}")
        
        # 提取所有 ```diff 代码块
        diff_blocks = re.findall(r'```diff\s+(.*?)```', text, re.DOTALL)
        
        for diff_block in diff_blocks:
            if diff_block is None or diff_block == '':
                continue
            # 尝试从 diff 块中提取文件路径
            file_paths = re.findall(r'(?:---|\+\+\+)\s+(?:a/|b/)?([^\n\t]+)', diff_block)
            
            if len(file_paths) >= 2:
                file_path_pre = file_paths[0]
                file_path_post = file_paths[1]
                last_file_path = file_path_post  # 更新最后使用的文件路径
            elif len(file_paths) == 1:
                file_path_pre = file_paths[0]
                file_path_post = file_paths[0]
                last_file_path = file_path_post  # 更新最后使用的文件路径
            else:
                # 如果没有找到文件路径，尝试其他格式
                git_diff_match = re.search(r'diff --git a/(.*?) b/(.*?)[\n\r]', diff_block)
                if git_diff_match:
                    file_path_pre = git_diff_match.group(1)
                    file_path_post = git_diff_match.group(2)
                    last_file_path = file_path_post  # 更新最后使用的文件路径
                elif last_file_path:
                    # 如果没有找到文件路径，但有前一个文件的路径，则使用前一个文件的路径
                    logger.info(f"未找到文件路径，使用前一个文件的路径: {last_file_path}")
                    file_path_pre = last_file_path
                    file_path_post = last_file_path
                else:
                    # 如果仍然没有找到，使用默认名称
                    logger.warning(f"无法从 diff 块中提取文件路径，使用默认名称")
                    file_path_pre = "unknown_file.txt"
                    file_path_post = "unknown_file.txt"
                    last_file_path = file_path_post  # 更新最后使用的文件路径
            
            logger.info(f"找到 diff: {file_path_pre} -> {file_path_post}")
            diffs.append((file_path_pre, file_path_post, f'''diff\n{diff_block}\n'''))
        
        # 如果没有找到 diff 块，尝试提取文件路径和内容
        if not diffs:
            # 尝试提取文件路径和内容
            file_blocks = re.findall(r'```(?:.*?)\n(?:# )?(?:File|文件):\s*([^\n]+)\n(.*?)```', text, re.DOTALL)
            
            for file_path, file_content in file_blocks:
                file_path = file_path.strip()
                logger.info(f"找到文件内容: {file_path}")
                last_file_path = file_path  # 更新最后使用的文件路径
                diffs.append((file_path, file_path, file_content))
        
        # 如果仍然没有找到，尝试匹配任何代码块
        if not diffs:
            code_blocks = re.findall(r'```(?:.*?)\n(.*?)```', text, re.DOTALL)
            
            for i, content in enumerate(code_blocks):
                # 尝试从内容中提取文件路径
                file_path_match = re.search(r'(?:File|文件):\s*([^\n]+)', content)
                if file_path_match:
                    file_path = file_path_match.group(1).strip()
                    logger.info(f"从代码块内容中提取到文件路径: {file_path}")
                    last_file_path = file_path  # 更新最后使用的文件路径
                    diffs.append((file_path, file_path, content))
                elif last_file_path:
                    # 如果没有找到文件路径，但有前一个文件的路径，则使用前一个文件的路径
                    logger.info(f"未从代码块中提取到文件路径，使用前一个文件的路径: {last_file_path}")
                    diffs.append((last_file_path, last_file_path, content))
                else:
                    logger.warning(f"找到代码块但无法提取文件路径，将使用默认文件名: code_block_{i}.txt")
                    file_path = f"code_block_{i}.txt"
                    last_file_path = file_path  # 更新最后使用的文件路径
                    diffs.append((file_path, file_path, content))
        
        if not diffs:
            logger.warning("未找到任何有效的 diff 或文件内容")
            # 打印文本的前200个字符，帮助调试
            logger.debug(f"文本前200个字符: {text[:200]}")
        else:
            logger.info(f"共找到 {len(diffs)} 个文件修改信息")
        
        return diffs

    @staticmethod
    def extract_raw_diff_blocks(text: str) -> List[str]:
        """
        从文本中提取所有原始的diff代码块，包含```diff ```标识

        Args:
            text: 包含文件修改信息的文本

        Returns:
            List[str]: 所有提取到的原始diff块列表，包含```diff ```和``` 标记
        """
        
        # 提取所有 ```diff 开头，以 ``` 结尾的代码块（包含标记）
        pattern = r'(```diff[\s\S]*?```)'  
        raw_diff_blocks = re.findall(pattern, text)
        
        if not raw_diff_blocks:
            logger.warning("未找到任何原始diff代码块")
                
        return raw_diff_blocks

    def process_diffs(self, diffs: List[Tuple[str, str, str]], project_dir: str) -> tuple[List[str], List[DiffInfo]]:
        """
        处理 diff 列表，使用 AI 模型生成新文件内容
        将同一个文件的多个 diff 合并后一起请求模型

        Args:
            diffs: 解析后的文件信息列表，每个元素为 (原文件路径, 新文件路径, diff内容)
            project_dir: 项目根目录

        Returns:
            List[str]: 处理失败的文件列表
        """
        failed_files = []
        diff_infos = []
        
        # 按文件路径分组，合并同一文件的多个 diff
        file_diffs = {}
        for file_path_pre, file_path_post, content_or_diff in diffs:
            if file_path_post not in file_diffs:
                file_diffs[file_path_post] = []
            file_diffs[file_path_post].append((file_path_pre, content_or_diff))
        
        logger.info(f"将 {len(diffs)} 个 diff 合并为 {len(file_diffs)} 个文件的修改")

        # 处理每个文件的所有 diff
        for file_path_post, file_changes in file_diffs.items():
            try:
                full_path_post = os.path.join(project_dir, file_path_post)
                
                # 获取第一个 diff 的原文件路径（通常所有 diff 的原文件路径应该相同）
                file_path_pre = file_changes[0][0]
                full_path_pre = os.path.join(project_dir, file_path_pre)
                
                # 检查是否是新文件
                is_new_file = file_path_pre == "/dev/null" or not os.path.exists(full_path_pre)
                
                # 合并同一文件的所有 diff
                combined_diff = "\n".join([change[1] for change in file_changes])
                info = DiffInfo()
                info.file_name = file_path_post
                if is_new_file:
                    # 对于新文件，直接使用内容创建
                    if combined_diff.startswith("diff ") or combined_diff.startswith("--- ") or combined_diff.startswith("+++ ") or "\n@@" in combined_diff:
                        # 如果是 diff 格式，需要让模型生成完整内容
                        prompt = f"""
                        我需要根据以下 diff 信息创建一个新文件。
                        
                        diff 信息：
                        ```
                        {combined_diff}
                        ```
                        
                        请生成完整的文件内容，然后使用 replace_file 工具将内容写入文件 {full_path_post}。
                        """
                    else:
                        # 如果已经是完整内容，直接使用
                        prompt = f"""
                        请使用 replace_file 工具将以下内容写入文件 {full_path_post}：
                        
                        ```
                        {combined_diff}
                        ```
                        """
                    info.content = combined_diff
                    info.is_create = True
                else:
                    # 对于现有文件，读取原内容，让模型生成新内容
                    try:
                        with open(full_path_pre, "r", encoding="utf-8") as f:
                            original_content = f.read()
                        
                        # 如果有多个 diff，清晰地标记每个 diff
                        if len(file_changes) > 1:
                            diff_sections = []
                            for i, (_, diff_content) in enumerate(file_changes, 1):
                                diff_sections.append(f"Diff #{i}:\n{diff_content}")
                            formatted_diffs = "\n\n".join(diff_sections)
                        else:
                            formatted_diffs = combined_diff
                        
                        prompt = f"""
                        我需要根据 diff 信息修改一个文件。原文件内容如下：
                        ```
                        {original_content}
                        ```
                        
                        {"以下是多个需要应用的 diff，请按顺序应用所有修改：" if len(file_changes) > 1 else "diff 信息如下："}
                        ```
                        {formatted_diffs}
                        ```
                        
                        请根据原文件内容和 diff 信息，生成修改后的完整文件内容，然后使用 replace_file 工具将内容写入文件 {full_path_post}。
                        {"请确保应用所有的 diff 修改，并解决可能的冲突。" if len(file_changes) > 1 else ""}
                        """
                        info.file_content = original_content
                        info.content = formatted_diffs
                        info.is_modify = True
                    except Exception as e:
                        logger.error(f"读取原文件失败: {str(e)}")
                        failed_files.append(file_path_post)
                        continue

                # 调用 AI 模型处理
                logger.info(f"处理文件: {file_path_post} (包含 {len(file_changes)} 个 diff)")
                response = self.ai_assistant.generate_response(prompt, use_tools=True)
                
                # 检查响应中是否包含成功信息
                if "文件已更新:" not in response:
                    logger.warning(f"文件处理可能失败: {file_path_post}, 响应: {response}")
                    failed_files.append(file_path_post)
                else:
                    diff_infos.append(info)
                    logger.info(f"处理文件成功: {file_path_post}")
            except Exception as e:
                logger.error(f"处理文件失败: {file_path_post}, 错误: {str(e)}")
                failed_files.append(file_path_post)
        
        return (failed_files, diff_infos)

    def _replace_file(self, file_path: str, content: str) -> str:
        """
        替换或创建UTF-8编码的文本文件
        支持自动创建不存在的目录结构
        """
        try:
            os.makedirs(os.path.dirname(file_path), exist_ok=True)
            with open(file_path, "w", encoding="utf-8") as f:
                f.write(content)
            return f"文件已更新: {file_path}"
        except UnicodeEncodeError:
            return "错误：内容包含非UTF-8字符"
        except Exception as e:
            return f"操作失败: {str(e)}"


    class _ReplaceFileSchema(BaseModel):
        file_path: str = Field(
            ...,
            examples=["data/config.json"],
            description="文件的相对目录"
        )
        content: str = Field(
            ...,
            examples=["""
    def calculate_sum(a, b):
        return a + b
    
    if __name__ == '__main__':
        print(calculate_sum(3, 5))
    """],
            description="UTF-8编码的文本内容，最大支持10MB"
        )

    def _create_replace_file_tool(self) -> StructuredTool:
        """创建替换文件内容的工具"""
        return StructuredTool.from_function(
            func=self._replace_file,
            name="replace_file",
            description="替换或创建文件内容。输入应为文件路径和文件内容。",
            args_schema=self._ReplaceFileSchema,
            return_direct=True,
        )


    def _get_diff_system_prompt(self) -> str:
        """
        获取处理 diff 的系统提示词

        Returns:
            str: 系统提示词
        """
        return """你是一个专业的代码工程师助手，擅长根据 diff 信息修改文件。
    
    你将收到原始文件内容和 diff 信息，需要生成修改后的完整文件内容。
    
    使用以下工具来完成任务：
    1. replace_file: 用于替换或创建文件内容
    
    处理步骤：
    1. 分析原始文件内容和 diff 信息
    2. 生成修改后的完整文件内容
    3. 使用 replace_file 工具将内容写入文件
    
    注意事项：
    - 确保生成的文件内容是完整的，包含所有必要的代码
    - 一定不可以删除原文件中的未修改内容
    - 不要添加额外的注释或标记
    - 只修改 diff 中指定的部分，保持其他部分不变
    - 如果是新文件，直接生成完整的文件内容
    """

if __name__ == "__main__":
    load_dotenv()
    text = '''
'''
    diff = Diff(AIConfig(temperature=0.1,
              model_name="gpt-4o"))
    diff.process_diffs(Diff.parse_diffs_from_text(text), "../.")



"""
File Memory API

Provides programmable API functions for using FileMemory without LogManager.
"""

import os
from typing import Dict, List, Optional

from dotenv import load_dotenv

from core.ai import AIConfig
from core.file_memory import FileMemory, FileMemoryConfig
from core.git_manager import GitManager, GitConfig


def init_file_memory(
    project_dir: str,
    model_name: str = "gpt-4o",
    temperature: float = 0.7,
    api_key: Optional[str] = None,
    base_url: Optional[str] = None,
    remote_url: Optional[str] = None,
    auth_token: Optional[str] = None,
) -> FileMemory:
    """
    Initialize a FileMemory instance with GitManager (no LogManager).
    
    Args:
        project_dir: Path to the project directory
        model_name: AI model to use for generating file descriptions
        temperature: Temperature setting for AI responses
        api_key: API key for AI service (will use env var if None)
        base_url: Base URL for AI service (will use default if None)
        remote_url: Git remote URL (will use env var if None)
        auth_token: Git authentication token (will use env var if None)
        
    Returns:
        Initialized FileMemory instance
    """
    # Load environment variables if not already loaded
    load_dotenv()
    
    # Create AI config
    ai_config = AIConfig(
        model_name=model_name,
        temperature=temperature,
        api_key=api_key,
        base_url=base_url
    )
    
    # Create Git config
    git_config = GitConfig(
        repo_path=project_dir,
        remote_url=remote_url or os.getenv("GIT_REMOTE"),
        auth_token=auth_token or os.getenv("GITHUB_TOKEN")
    )
    
    # Initialize Git manager
    git_manager = GitManager(config=git_config)
    
    # Initialize and return FileMemory
    file_memory_config = FileMemoryConfig(
        project_dir=project_dir,
        git_manager=git_manager,
        ai_config=ai_config,
        log_manager=None  # Explicitly None as per requirements
    )
    
    return FileMemory(config=file_memory_config)


def update_file_descriptions(file_memory: FileMemory) -> None:
    """
    Update file descriptions using the given FileMemory instance.
    
    Args:
        file_memory: Initialized FileMemory instance
    """
    file_memory.update_file_details()


def process_failed_files(file_memory: FileMemory) -> Dict[str, str]:
    """
    Process previously failed files to generate their descriptions.
    
    Args:
        file_memory: Initialized FileMemory instance
        
    Returns:
        Dictionary mapping file paths to their descriptions
    """
    return file_memory.process_failed_files()



"""
Command-line interface for the WorkflowEngine.
Provides functionality to parse command-line arguments and run the engine.
"""

import argparse
import os
import sys
from typing import Optional, Dict, Any


def parse_args() -> argparse.Namespace:
    """Parse command-line arguments for the WorkflowEngine."""
    parser = argparse.ArgumentParser(
        description="Run the WorkflowEngine to process user requirements"
    )

    # Required arguments
    parser.add_argument(
        "--project-dir", 
        "-p",
        type=str, 
        default=os.path.abspath(os.getcwd()),
        help="Path to the project directory (default: current directory)"
    )
    parser.add_argument(
        "--issue-id", 
        "-i",
        type=int, 
        required=True,
        help="The ID of the issue being processed"
    )
    parser.add_argument(
        "--requirement", 
        "-r",
        type=str, 
        help="The user requirement text"
    )
    parser.add_argument(
        "--requirement-file", 
        "-f",
        type=str, 
        help="Path to file containing the user requirement"
    )

    # Optional arguments for WorkflowEngineConfig
    # 统一模型配置
    parser.add_argument(
        "--model", 
        "-m",
        type=str, 
        help="Model to use for both core and data operations (优先级高于单独配置)"
    )
    parser.add_argument(
        "--temperature",
        "-t",
        type=float, 
        help="Temperature for both core and data models (优先级高于单独配置)"
    )
    
    # 独立模型配置
    parser.add_argument(
        "--core-model", 
        "--cm",
        type=str, 
        default="gpt-4o",
        help="Model to use for core AI operations (当未设置--model时使用)"
    )
    parser.add_argument(
        "--data-model", 
        "--dm",
        type=str, 
        default="gpt-4o",
        help="Model to use for data operations (当未设置--model时使用)"
    )
    parser.add_argument(
        "--core-temperature",
        "--ct",
        type=float, 
        default=0.7,
        help="Temperature for core model (当未设置--temperature时使用)"
    )
    parser.add_argument(
        "--data-temperature",
        "--dt",
        type=float, 
        default=0.7,
        help="Temperature for data model (当未设置--temperature时使用)"
    )
    parser.add_argument(
        "--max-retry", 
        "--retry",
        type=int, 
        default=3,
        help="Maximum number of retry attempts"
    )
    parser.add_argument(
        "--default-branch", 
        "--branch",
        type=str, 
        default="main",
        help="Default branch name"
    )
    parser.add_argument(
        "--mode",
        "-md",
        type=str,
        choices=["client", "bot"],
        default="client",
        help="Operation mode: 'client' or 'bot'"
    )
    parser.add_argument(
        "--base-url",
        "-u",
        type=str,
        help="Base URL for API calls"
    )
    parser.add_argument(
        "--api-key",
        "-k",
        type=str,
        help="API key for authentication"
    )
    parser.add_argument(
        "--github-remote-url",
        "--git-url",
        type=str,
        help="GitHub remote repository URL"
    )
    parser.add_argument(
        "--github-token",
        "--git-token",
        type=str,
        help="GitHub authentication token"
    )
    
    return parser.parse_args()


def get_requirement_text(args: argparse.Namespace) -> Optional[str]:
    """Get requirement text from arguments or file."""
    if args.requirement:
        return args.requirement
    elif args.requirement_file:
        try:
            with open(args.requirement_file, 'r', encoding='utf-8') as file:
                return file.read()
        except IOError as e:
            print(f"Error reading requirement file: {e}", file=sys.stderr)
            return None
    else:
        print("No requirement specified. Use --requirement or --requirement-file", file=sys.stderr)
        return None


def build_config_from_args(args: argparse.Namespace) -> Dict[str, Any]:
    """Build WorkflowEngineConfig parameters from command line arguments."""
    
    # 处理统一的模型和温度配置
    core_model = args.core_model
    data_model = args.data_model
    core_temperature = args.core_temperature
    data_temperature = args.data_temperature
    
    # 如果设置了统一模型，则覆盖个别设置
    if args.model:
        core_model = args.model
        data_model = args.model
        
    # 如果设置了统一温度，则覆盖个别设置
    if args.temperature is not None:
        core_temperature = args.temperature
        data_temperature = args.temperature
    
    config_params = {
        "project_dir": args.project_dir,
        "issue_id": args.issue_id,
        "core_model": core_model,
        "data_model": data_model,
        "core_template": core_temperature,  # Note: using template to match original param name
        "data_template": data_temperature,  # Note: using template to match original param name
        "max_retry": args.max_retry, 
        "default_branch": args.default_branch,
        "mode": args.mode,
    }
    
    # Add optional parameters if they're specified
    if args.base_url:
        config_params["base_url"] = args.base_url
    if args.api_key:
        config_params["api_key"] = args.api_key
    if args.github_remote_url:
        config_params["github_remote_url"] = args.github_remote_url
    if args.github_token:
        config_params["github_token"] = args.github_token
        
    return config_params



import os
import pathlib
from typing import Set

from pathspec import PathSpec
from pathspec.patterns import GitWildMatchPattern


class FileFetcher:
    """Manages file operations and selections for the project"""

    @staticmethod
    def read_gitignore(root_dir: str) -> PathSpec:
        """Read .gitignore file and create a PathSpec for pattern matching"""
        gitignore_path = os.path.join(root_dir, ".gitignore")
        patterns = []

        if os.path.exists(gitignore_path):
            with open(gitignore_path, "r", encoding="utf-8") as f:
                for line in f:
                    line = line.strip()
                    if line and not line.startswith("#"):
                        patterns.append(line)

        gpteng_path = os.path.join(root_dir, ".eng", ".engignore")

        if os.path.exists(gpteng_path):
            with open(gpteng_path, "r", encoding="utf-8") as f:
                for line in f:
                    line = line.strip()
                    if line and not line.startswith("#"):
                        patterns.append(line)
        return PathSpec.from_lines(GitWildMatchPattern, patterns)

    @staticmethod
    def get_all_files(root_dir: str, gitignore_spec: PathSpec) -> Set[str]:
        """Get all files in directory that aren't ignored by .gitignore"""
        all_files = set()
        root_path = pathlib.Path(root_dir)

        for path in root_path.rglob("*"):
            if path.is_file():
                # 转换为相对路径
                relative_path = str(path.relative_to(root_path))

                # 跳过匹配 gitignore 模式的文件
                if not gitignore_spec.match_file(relative_path):
                    all_files.add(relative_path)

        return all_files

    @staticmethod
    def get_all_files_without_ignore(root_dir: str) -> Set[str]:
        # Read gitignore patterns
        gitignore_spec = FileFetcher.read_gitignore(root_dir)

        # Get all files
        return FileFetcher.get_all_files(root_dir, gitignore_spec)


if __name__ == "__main__":
    # Example usage
    print(FileFetcher.get_all_files_without_ignore(".."))



import os
import shutil
from dataclasses import dataclass
from typing import Optional, List
from urllib.parse import urlparse, urlunparse

import git

from core.log_config import get_logger

logger = get_logger(__name__)


@dataclass
class GitConfig:
    """Git configuration parameters"""

    repo_path: str
    remote_name: str = "origin"
    default_branch: str = "main"
    remote_url: Optional[str] = os.getenv("GIT_REMOTE")
    auth_token: Optional[str] = os.getenv("GITHUB_TOKEN")


class GitManager:
    """Manages git operations including push, pull, branch creation and switching"""

    def __init__(self, config: GitConfig):
        """Initialize GitManager with configuration"""
        self.config = config
        self.repo = None
        self._ensure_repo()

    def _ensure_repo(self) -> None:
        """Ensure git repository exists and is properly initialized"""
        if not os.path.exists(self.config.repo_path) or len(os.listdir(self.config.repo_path)) == 0:
            if self.config.remote_url:
                self.clone()
            else:
                raise ValueError(
                    f"Repository path does not exist: {self.config.repo_path}"
                )
        else:
            try:
                self.repo = git.Repo(self.config.repo_path)
                # Set auth token for remote operations if provided
                if self.config.auth_token and self.config.remote_url:
                    self._set_remote_with_auth()
            except git.InvalidGitRepositoryError:
                raise ValueError(
                    f"Invalid git repository at: {self.config.repo_path}"
                )



    def _get_url_with_token(self, url: str) -> str:
        """
        Insert authentication token into git URL

        Args:
            url: Original git URL

        Returns:
            URL with authentication token
        """
        if not self.config.auth_token:
            return url

        parsed = urlparse(url)

        # Handle different URL formats
        if parsed.scheme in ["http", "https"]:
            netloc = f"{self.config.auth_token}@{parsed.netloc}"
            return urlunparse(
                (
                    parsed.scheme,
                    netloc,
                    parsed.path,
                    parsed.params,
                    parsed.query,
                    parsed.fragment,
                )
            )
        elif "@" in url and ":" in url and url.startswith("git@"):  # Handle SSH format: git@github.com:username/repo.git
            return url  # Don't modify SSH URLs
        else:
            return url  # Return original if format is not recognized

    def _set_remote_with_auth(self) -> None:
        """Configure remote with authentication token"""
        if not self.repo or not self.config.auth_token or not self.config.remote_url:
            return

        try:
            # Get current remotes
            remotes = list(self.repo.remotes)
            remote_exists = any(
                remote.name == self.config.remote_name for remote in remotes
            )

            # Prepare URL with token
            url_with_token = self._get_url_with_token(self.config.remote_url)

            # Set or update remote
            if remote_exists:
                self.repo.git.remote("set-url", self.config.remote_name, url_with_token)
            else:
                self.repo.git.remote("add", self.config.remote_name, url_with_token)

            logger.info(
                f"Configured remote '{self.config.remote_name}' with authentication"
            )
        except git.GitCommandError as e:
            logger.error(f"Failed to configure remote with authentication: {str(e)}")
            raise

    def clone(self) -> None:
        """
        Clone the repository specified in config

        Raises:
            ValueError: If remote_url is not set in config
            git.GitCommandError: If clone operation fails
        """
        if not self.config.remote_url:
            raise ValueError("Remote URL must be set to clone a repository")

        self._clone_repo(branch=self.config.default_branch)

    def _clone_repo(self, branch: Optional[str] = None) -> None:
        """
        Internal method to perform the clone operation

        Args:
            branch: Branch to checkout after cloning
        """
        try:
            # Prepare parent directory if it doesn't exist
            parent_dir = os.path.dirname(self.config.repo_path)
            if parent_dir and not os.path.exists(parent_dir):
                os.makedirs(parent_dir)

            # Remove target directory if it exists
            if os.path.exists(self.config.repo_path):
                shutil.rmtree(self.config.repo_path)

            # Prepare URL with token if provided
            clone_url = self._get_url_with_token(self.config.remote_url)

            # Clone options
            clone_args = {
                "url": clone_url,
                "to_path": self.config.repo_path,
            }

            # Add branch if specified
            if branch:
                clone_args["branch"] = branch

            # Clone the repository
            self.repo = git.Repo.clone_from(**clone_args)

            logger.info(f"Successfully cloned repository to {self.config.repo_path}")

            # Configure remote with auth token if provided
            if self.config.auth_token:
                self._set_remote_with_auth()

        except git.GitCommandError as e:
            logger.error(f"Failed to clone repository: {str(e)}")
            # Clean up if clone failed
            if os.path.exists(self.config.repo_path):
                shutil.rmtree(self.config.repo_path)
            raise

    def pull(self, branch: Optional[str] = None) -> None:
        """
        Pull changes from remote repository

        Args:
            branch: Branch to pull from. If None, pulls current branch
        """
        try:
            if branch:
                self.repo.git.pull(self.config.remote_name, branch)
            else:
                self.repo.git.pull()
            logger.info(
                f"Successfully pulled changes from {branch or 'current branch'}"
            )
        except git.GitCommandError as e:
            logger.error(f"Failed to pull changes: {str(e)}")
            raise

    def push(
        self,
        branch: Optional[str] = None,
        force: bool = False,
        set_upstream: bool = True,
    ) -> None:
        """
        Push changes to remote repository

        Args:
            branch: Branch to push. If None, pushes current branch
            force: Whether to force push
            set_upstream: Whether to set upstream branch if it doesn't exist
        """
        try:
            # 确保远程 URL 包含认证令牌
            if self.config.auth_token:
                self._set_remote_with_auth()

            # 如果没有指定分支，获取当前分支
            current_branch = branch or self.get_current_branch()

            # 执行推送操作
            if force:
                if set_upstream:
                    self.repo.git.push(
                        "-f", "--set-upstream", self.config.remote_name, current_branch
                    )
                else:
                    if branch:
                        self.repo.git.push("-f", self.config.remote_name, branch)
                    else:
                        self.repo.git.push("-f")
            else:
                if set_upstream:
                    self.repo.git.push(
                        "--set-upstream", self.config.remote_name, current_branch
                    )
                else:
                    if branch:
                        self.repo.git.push(self.config.remote_name, branch)
                    else:
                        self.repo.git.push()

            logger.info(f"Successfully pushed changes to {current_branch}")
        except git.GitCommandError as e:
            logger.error(f"Failed to push changes: {str(e)}")
            raise

    def create_branch(
        self, branch_name: str, start_point: Optional[str] = None
    ) -> None:
        """
        Create a new branch

        Args:
            branch_name: Name of the new branch
            start_point: Branch/commit to create branch from. If None, uses current HEAD
        """
        try:
            if start_point:
                self.repo.git.branch(branch_name, start_point)
            else:
                self.repo.git.branch(branch_name)
            logger.info(f"Successfully created branch: {branch_name}")
        except git.GitCommandError as e:
            logger.error(f"Failed to create branch: {str(e)}")
            raise

    def switch_branch(self, branch_name: str, create: bool = False) -> None:
        """
        Switch to specified branch

        Args:
            branch_name: Name of the branch to switch to
            create: Create branch if it doesn't exist
        """
        try:
            if create:
                try:
                    self.repo.git.checkout("-b", branch_name)
                except:
                    self.repo.git.checkout(branch_name)
            else:
                self.repo.git.checkout(branch_name)
            logger.info(f"Successfully switched to branch: {branch_name}")
        except git.GitCommandError as e:
            logger.error(f"Failed to switch branch: {str(e)}")
            raise

    def delete_branch(self, branch_name: str, force: bool = False) -> None:
        """
        Delete specified branch

        Args:
            branch_name: Name of the branch to delete
            force: Force delete even if branch not fully merged
        """
        try:
            if force:
                self.repo.git.branch("-D", branch_name)
            else:
                self.repo.git.branch("-d", branch_name)
            logger.info(f"Successfully deleted branch: {branch_name}")
        except git.GitCommandError as e:
            logger.error(f"Failed to delete branch: {str(e)}")
            raise

    def get_current_branch(self) -> str:
        """Get name of current branch"""
        return self.repo.active_branch.name

    def list_branches(self, remote: bool = False) -> List[str]:
        """
        List all branches

        Args:
            remote: Whether to list remote branches instead of local

        Returns:
            List of branch names
        """
        if remote:
            return [ref.name for ref in self.repo.remote().refs]
        return [branch.name for branch in self.repo.heads]

    def get_current_commit_id(self) -> str:
        """获取当前提交的 ID"""
        try:
            return self.repo.head.commit.hexsha
        except Exception as e:
            logger.error(f"获取当前提交 ID 失败: {str(e)}")
            return ""

    def get_changed_files(self, old_commit: str, new_commit: str) -> List[str]:
        """
        获取两个提交之间变更的文件列表

        Args:
            old_commit: 旧提交的 ID
            new_commit: 新提交的 ID

        Returns:
            变更的文件路径列表
        """
        try:
            # 获取提交对象
            old = self.repo.commit(old_commit)
            new = self.repo.commit(new_commit)

            # 获取差异
            diff_index = old.diff(new)

            # 收集所有变更的文件
            changed_files = set()

            # 添加修改的文件
            for diff in diff_index.iter_change_type("M"):
                if not self.is_ignore(diff.a_path):
                    changed_files.add(diff.a_path)

            # 添加增加的文件
            for diff in diff_index.iter_change_type("A"):
                if not self.is_ignore(diff.b_path):
                    changed_files.add(diff.b_path)

            # 添加删除的文件
            for diff in diff_index.iter_change_type("D"):
                if not self.is_ignore(diff.b_path):
                    changed_files.add(diff.a_path)

            return list(changed_files)
        except Exception as e:
            logger.error(f"获取变更文件列表失败: {str(e)}")
            return []

    def is_ignore(self, path: str) -> bool:
        # 检查文件名是否以点开头
        file_name = os.path.basename(path)
        if file_name.startswith("."):
            return True
        # 检查路径中是否包含以点开头的目录
        path_parts = path.split(os.path.sep)
        for part in path_parts:
            # 跳过空字符串（可能出现在路径开头）
            if not part:
                continue
            # 如果目录名以点开头，则忽略
            if part.startswith("."):
                return True

        # 如果不满足任何忽略条件，则不忽略
        return False

    def delete_local_repository(self, remove_git_config: bool = False) -> None:
        """
        删除本地代码仓库和可选的全局 git 配置

        Args:
            remove_git_config: 是否同时删除与此仓库相关的全局 git 配置

        Raises:
            ValueError: 如果仓库路径不存在
            OSError: 如果删除操作失败
        """
        if not os.path.exists(self.config.repo_path):
            logger.warning(f"仓库路径不存在，无需删除: {self.config.repo_path}")
            return

        try:
            # 关闭仓库连接以释放文件锁
            if self.repo:
                self.repo.close()
                self.repo = None

            # 删除本地仓库目录
            shutil.rmtree(self.config.repo_path)
            logger.info(f"成功删除本地仓库: {self.config.repo_path}")

            # 可选：删除全局 git 配置中与此仓库相关的条目
            if remove_git_config:
                self._remove_git_config()

        except (OSError, shutil.Error) as e:
            logger.error(f"删除本地仓库失败: {str(e)}")
            raise

    def _remove_git_config(self) -> None:
        """
        从全局 git 配置中删除与当前仓库相关的配置

        这包括：
        - 与远程仓库 URL 相关的凭证
        - 特定于此仓库的用户配置
        """
        try:
            # 获取仓库的规范路径
            repo_path = os.path.abspath(self.config.repo_path)
            repo_name = os.path.basename(repo_path)

            # 尝试从 git 配置中删除与此仓库相关的条目
            if self.config.remote_url:
                # 解析远程 URL 以获取主机名
                parsed = urlparse(self.config.remote_url)
                if parsed.netloc:
                    # 尝试删除凭证
                    try:
                        git.cmd.Git().execute(
                            ["git", "credential", "reject"],
                            input=f"url={self.config.remote_url}\n\n",
                        )
                        logger.info(f"已尝试从凭证存储中删除 {parsed.netloc} 的凭证")
                    except git.GitCommandError:
                        logger.debug("凭证删除操作未成功，可能没有存储凭证")

            # 尝试删除仓库特定的配置（如果有）
            try:
                git.cmd.Git().execute(
                    [
                        "git",
                        "config",
                        "--global",
                        "--remove-section",
                        f"remote.{repo_name}",
                    ]
                )
                logger.info(f"已删除全局 git 配置中的 remote.{repo_name} 部分")
            except git.GitCommandError:
                logger.debug(f"全局配置中没有 remote.{repo_name} 部分")

            logger.info("已完成 git 配置清理")

        except Exception as e:
            logger.warning(f"清理 git 配置时出错: {str(e)}")
            # 不抛出异常，因为这是次要操作

    def add_issue_comment(self, issue_number: int, comment_text: str) -> bool:
        """
        在 GitHub 仓库的指定 Issue 下添加评论

        Args:
            issue_number: Issue 编号
            comment_text: 评论内容

        Returns:
            bool: 操作是否成功

        Raises:
            ValueError: 如果未配置认证令牌或远程 URL
            Exception: 如果添加评论过程中发生其他错误
        """
        try:
            # 检查必要的配置
            if not self.config.auth_token:
                raise ValueError("添加 Issue 评论需要认证令牌 (auth_token)")

            if not self.config.remote_url:
                raise ValueError("添加 Issue 评论需要远程仓库 URL (remote_url)")

            # 导入 PyGithub
            try:
                from github import Github
            except ImportError:
                raise ImportError("添加 Issue 评论需要安装 PyGithub 库: pip install PyGithub")

            # 解析仓库所有者和名称
            parsed_url = urlparse(self.config.remote_url)
            path_parts = parsed_url.path.strip("/").split("/")

            if len(path_parts) < 2 or not all(path_parts[:2]):
                raise ValueError(f"无法从 URL 解析仓库所有者和名称: {self.config.remote_url}")

            owner = path_parts[0]
            repo_name = path_parts[1]
            if repo_name.endswith(".git"):
                repo_name = repo_name[:-4]  # 移除 .git 后缀

            # 初始化 GitHub 客户端
            g = Github(self.config.auth_token)

            # 获取仓库和 Issue
            repo = g.get_repo(f"{owner}/{repo_name}")
            issue = repo.get_issue(issue_number)

            # 添加评论
            comment = issue.create_comment(f"bella-issues-bot已处理：\n{comment_text}")

            logger.info(f"成功在 Issue #{issue_number} 下添加评论 (ID: {comment.id})")
            return True

        except Exception as e:
            logger.error(f"添加 Issue 评论失败: {str(e)}")
            raise

    def commit(
        self, message: str, add_all: bool = True, files: Optional[List[str]] = None
    ) -> str:
        """
        创建一个新的提交

        Args:
            message: 提交信息
            add_all: 是否添加所有变更的文件，默认为 True
            files: 要添加的特定文件列表，如果 add_all 为 True 则忽略此参数

        Returns:
            str: 新提交的 SHA 哈希值

        Raises:
            git.GitCommandError: 如果 Git 操作失败
        """
        try:
            # 添加文件到暂存区
            if add_all:
                self.repo.git.add(A=True)
            elif files:
                for file in files:
                    self.repo.git.add(file)

            # 创建提交
            commit = self.repo.index.commit(message)
            logger.info(f"成功创建提交: {commit.hexsha[:7]} - {message}")

            return commit.hexsha
        except git.GitCommandError as e:
            logger.error(f"创建提交失败: {str(e)}")
            raise

    def reset_to(self, target_branch: str) -> bool:
        """
        将当前分支重置到远程目标分支的状态

        Args:
            target_branch: 目标分支名称

        Returns:
            bool: 操作是否成功
        """
        try:
            # 获取远程分支
            remote_name = self.config.remote_name
            remote_branches = self.list_branches(remote=True)
            remote_target = f"{remote_name}/{target_branch}"
            # 检查目标分支是否存在于远端
            if remote_target not in remote_branches:
                logger.warning(f"目标分支 {remote_target} 不存在于远端")
                return False

            # 切换到目标分支，如果不存在则创建
            logger.info(f"切换到分支: {target_branch}")
            self.switch_branch(target_branch, create=True)

            # 拉取最新代码
            logger.info(f"拉取远程分支: {remote_target} 的最新代码")
            self.pull(target_branch)

            # 强制重置到远程分支状态
            logger.info(f"重置到远程分支: {remote_target}")
            self.repo.git.reset(f"{remote_target}", hard=True)

            logger.info(f"成功重置到版本: {target_branch}")
            return True
        except git.GitCommandError as e:
            logger.error(f"重置到分支 {target_branch} 失败: {str(e)}")
            return False
        except Exception as e:
            logger.error(f"重置过程中发生未知错误: {str(e)}")
            return False

    def reset_to_issue_branch(self, issue_id: int) -> str:
        """
        拉取指定issue对应的最新分支并切换到该分支
        如果该issue还未创建过分支，则切换到默认分支

        Args:
            issue_id: Issue编号

        Returns:
            str: 成功切换到的分支名称

        Raises:
            git.GitCommandError: 如果Git操作失败
        """
        try:
            # 确保远程仓库信息是最新的
            self.repo.git.fetch(self.config.remote_name)
            logger.info(f"成功获取远程仓库信息")
            
            # 获取所有远程分支
            remote_branches = self.repo.git.branch("-r").splitlines()
            remote_branches = [branch.strip() for branch in remote_branches]
            
            # 查找与指定issue相关的分支
            issue_branch_name = f"bella-issues-bot-{issue_id}"
            remote_issue_branch = f"{self.config.remote_name}/{issue_branch_name}"
            
            branch_exists = False
            for branch in remote_branches:
                if remote_issue_branch in branch:
                    branch_exists = True
                    break
                    
            if branch_exists:
                # 切换到issue分支
                self.switch_branch(issue_branch_name, create=True)
                self.pull()
                logger.info(f"成功切换到issue #{issue_id}的分支: {issue_branch_name}")
            else:
                # 如果没有找到相关分支，切换到默认分支
                self.switch_branch(self.config.default_branch)
                self.pull()
                logger.info(f"重置到远程分支: {self.config.remote_name}/{self.config.default_branch}")
                self.repo.git.reset(f"{self.config.remote_name}/{self.config.default_branch}", hard=True)
                logger.info(f"未找到issue #{issue_id}的分支，已切换到默认分支: {self.config.default_branch}")
            
            return issue_branch_name
        except git.GitCommandError as e:
            logger.error(f"切换到issue分支时出错: {str(e)}")
            raise



"""Client package for running the WorkflowEngine from the terminal."""

# Export file memory functions for programmatic use
from client.file_memory_client import initialize_file_memory, update_file_descriptions, process_failed_files
from client.github_workflow_generator import generate_workflow_files



from dataclasses import dataclass
from typing import Any, List, Optional

from dotenv import load_dotenv
from langchain.agents import AgentExecutor, create_openai_tools_agent
from langchain.callbacks.streaming_stdout import StreamingStdOutCallbackHandler
from langchain_core.prompts import ChatPromptTemplate, MessagesPlaceholder
from langchain_core.runnables.base import RunnableSequence
from langchain_core.tools import BaseTool, Tool
from langchain_openai import ChatOpenAI

from core.log_config import get_logger

logger = get_logger(__name__)


@dataclass
class AIConfig:
    model_name: str = "gpt-4o"
    temperature: float = 0.7
    verbose: bool = True
    max_retries: int = 3
    request_timeout: int = 180
    sys_prompt: str = "You are a helpful AI assistant."
    base_url: Optional[str] = None
    api_key: Optional[str] = None


class AIAssistant:
    """AI 助手类，负责与 AI 模型交互"""

    def __init__(self, config: AIConfig, tools: Optional[List[BaseTool]] = None):
        """
        初始化 AI 助手

        Args:
            config: AIConfig 实例，包含必要的配置信息
            tools: 可选的工具列表
        """
        self.config = config
        self.tools = tools or []
        self.llm = self._init_llm()
        self.agent = None

        # Initialize agent if tools are provided
        if self.tools:
            self.agent = self._init_agent()

    def _init_llm(self) -> ChatOpenAI:
        """Initialize the language model"""
        callbacks = [StreamingStdOutCallbackHandler()] if self.config.verbose else None
        
        return ChatOpenAI(
            base_url=self.config.base_url,
            api_key=self.config.api_key,
            model=self.config.model_name,
            temperature=self.config.temperature,
            timeout=self.config.request_timeout,
            max_retries=self.config.max_retries,
            callbacks=callbacks,
        )

    def _init_agent(self) -> AgentExecutor:
        """Initialize the agent with tools"""
        # 创建提示模板
        prompt = ChatPromptTemplate.from_messages([
            ("system", self.config.sys_prompt),
            ("human", "{input}"),
            MessagesPlaceholder(variable_name="agent_scratchpad"),
        ])

        # 创建代理
        agent = create_openai_tools_agent(self.llm, self.tools, prompt)
        
        # 创建代理执行器
        return AgentExecutor(
            agent=agent, 
            tools=self.tools, 
            verbose=self.config.verbose,
            max_iterations=5,
            handle_parsing_errors=True
        )

    def _create_simple_chain(self) -> RunnableSequence:
        """创建简单的对话链，不使用工具"""
        from langchain_core.prompts import ChatPromptTemplate
        
        # 创建提示模板
        prompt = ChatPromptTemplate.from_messages([
            ("system", self.config.sys_prompt),
            ("human", "{input}")
        ])
        
        # 创建简单链
        chain = prompt | self.llm | (lambda x: x.content)
        
        return chain

    def add_tool(self, tool: BaseTool) -> None:
        """
        添加工具

        Args:
            tool: 要添加的工具
        """
        # 检查是否已经有同名工具
        for existing_tool in self.tools:
            if existing_tool.name == tool.name:
                # 替换同名工具
                self.tools.remove(existing_tool)
                break
                
        # 添加新工具
        self.tools.append(tool)
        
        # 重新初始化代理
        self.agent = self._init_agent()

    def generate_response(
        self, prompt: str, use_tools: bool = False, **kwargs: Any
    ) -> Any:
        """
        生成响应

        Args:
            prompt: 用户的提示词
            use_tools: 是否使用工具
            **kwargs: 其他参数

        Returns:
            any: 生成的响应
        """
        try:
            if use_tools and self.tools:
                # 确保代理已初始化
                if self.agent is None:
                    self.agent = self._init_agent()
                    
                # 使用代理生成响应
                response = self.agent.invoke({"input": prompt})
                return response["output"]
            else:
                # 使用简单链生成响应，始终使用流式输出
                chain = self._create_simple_chain()
                
                # 使用流式输出
                response_chunks = []
                for chunk in chain.stream({"input": prompt}):
                    response_chunks.append(chunk)
                
                # response_chunks 连接起来就是完整的响应结果
                return "".join(response_chunks)
        except Exception as e:
            logger.error(f"生成响应时出错: {str(e)}")
            raise


def create_example_tool() -> Tool:
    """Create an example tool for demonstration"""

    def calculator(expression: str) -> str:
        try:
            result = eval(expression)
            return f"计算结果: {result}"
        except Exception as e:
            logger.error(f"计算错误: {str(e)}")
            return f"计算错误: {str(e)}"

    return Tool.from_function(
        func=calculator,
        name="calculator",
        description="计算数学表达式",
    )


if __name__ == "__main__":
    # 设置环境变量
    load_dotenv()
    
    # 创建AI助手
    ai_assistant = AIAssistant(
        config=AIConfig(model_name="gpt-4o", temperature=0.7)
    )
    
    # 添加示例工具
    ai_assistant.add_tool(create_example_tool())
    
    # 测试生成响应
    response = ai_assistant.generate_response(
        "计算 3 + 5 的结果是多少?", use_tools=True
    )
    
    logger.info(f"Response: {response}")



import os
from copy import copy
from dataclasses import dataclass, field

from dotenv import load_dotenv
from typing_extensions import Optional

from core.ai import AIAssistant, AIConfig
from core.diff import Diff
from core.log_manager import LogManager, LogConfig
from core.log_config import get_logger

logger = get_logger(__name__)

@dataclass
class CodeEngineerConfig:
    """代码工程师配置"""
    project_dir: str
    ai_config: AIConfig
    system_prompt: Optional[str] = None
    max_retries: int = 3


class CodeEngineer:
    """
    代码工程师类，负责处理用户的 prompt，与 AI 模型交互，解析 diff 并修改文件
    """

    def __init__(self, config: CodeEngineerConfig, log_manager: LogManager, diff: Diff):
        """
        初始化代码工程师

        Args:
            config: CodeEngineerConfig 实例，包含必要的配置信息
            log_manager: LogManager 实例，用于日志管理
        """
        self.config = config
        self.log_manager = log_manager
        self.diff = diff


        if config.system_prompt:
            self.system_prompt = config.system_prompt
        else:
            # 读取系统提示词
            self.system_prompt = self._read_system_prompt()

        self.ai_config = copy(config.ai_config)
        self.ai_config.sys_prompt = self.system_prompt
        self.ai_assistant = AIAssistant(config=self.ai_config)
        # 用于存储处理失败的文件
        self.failed_files = []
        # 用于存储修改的文件
        self.modified_files = []

    def _read_system_prompt(self) -> str:
        """
        读取系统提示词

        Returns:
            str: 系统提示词内容
        """
        try:
            system_prompt_path = os.path.join(self.config.project_dir, "system.txt")
            if os.path.exists(system_prompt_path):
                with open(system_prompt_path, "r", encoding="utf-8") as f:
                    return f.read()
            else:
                logger.warning(f"系统提示词文件不存在: {system_prompt_path}，使用默认提示词")
                return "You will get instructions for code to write. Output requested code changes in the unified git diff syntax."
        except Exception as e:
            logger.error(f"读取系统提示词失败: {str(e)}")
            return "You will get instructions for code to write. Output requested code changes in the unified git diff syntax."

    def process_prompt(self, prompt: str) ->  tuple[bool, Optional[str]]:
        """
        处理用户的 prompt，与 AI 模型交互，解析 diff 并修改文件

        Args:
            prompt: 用户的 prompt

        Returns:
            bool: 处理是否成功
            str: 模型返回结果
        """
        try:
            # 重置失败文件列表
            self.failed_files = []
            # 重置修改的文件列表
            self.diff_infos = []
            # 设置 AI 助手的系统提示词
            self.ai_assistant.config.sys_prompt = self.system_prompt
            
            # 调用 AI 模型生成响应
            response = self.ai_assistant.generate_response(prompt)
            
            # 解析响应中的 diff
            diffs = Diff.parse_diffs_from_text(response)
            
            if not diffs:
                logger.warning("未找到有效的 diff")
                return (False, None)
            
            # 处理每个 diff
            self.failed_files, self.diff_infos  = self.diff.process_diffs(diffs, self.config.project_dir)
            
            # 归档日志
            self.log_manager.archive_logs(
                sys_prompt=self.system_prompt,
                prompt=prompt,
                response=response,
                diff_infos=self.diff_infos
            )
            
            # 记录日志
            if self.modified_files:
                logger.info(f"本次修改了 {len(self.modified_files)} 个文件: " + 
                           ", ".join(self.modified_files[:5]) + 
                           ("..." if len(self.modified_files) > 5 else ""))
            
            # 如果有失败的文件，可以在这里处理
            if self.failed_files:
                logger.warning(f"有 {len(self.failed_files)} 个文件处理失败")
                # 这里可以添加失败文件的重试逻辑，但根据需求，暂时不实现
                return (False, response)
            
            return (True, response)
        except Exception as e:
            logger.error(f"处理 prompt 失败: {str(e)}")
            return (False, None)

    def retry_failed_files(self, ) -> bool:
        """
        重试处理失败的文件（钩子方法，暂不实现具体逻辑）

        Args:
            prompt: 用户的 prompt

        Returns:
            bool: 重试是否成功
        """
        # 这是一个钩子方法，用于未来扩展
        # 根据需求，暂时不实现具体逻辑

        return False

if __name__ == "__main__":
    load_dotenv()
    prompt = '''
    '''
    config = CodeEngineerConfig(project_dir="../.", ai_config=AIConfig(
        temperature=1,
        model_name="claude-3.7-sonnet"
    ))

    engineer = CodeEngineer(config, LogManager(LogConfig("../.", 1)), Diff(AIConfig(temperature=0.1,
                                                                                 model_name="gpt-4o")))
    engineer.process_prompt(prompt)



"""
版本管理模块，处理代码生成的版本控制、历史追踪和回退功能。

该模块提供了以下功能:
1. 提取历史轮次的日志信息
2. 格式化历史执行记录用于AI参考
3. 分析用户需求，决定是否需要版本回退
4. 执行Git版本回退操作
5. 为AI助手提供版本回退工具
"""

import os
from dataclasses import dataclass
from typing import List, Tuple, Optional

from langchain_core.tools import Tool, StructuredTool
from pydantic import BaseModel, Field

from core.ai import AIConfig, AIAssistant
from core.git_manager import GitManager
from core.log_config import get_logger
from core.log_manager import LogManager
from core.prompt_generator import PromptGenerator

logger = get_logger(__name__)

@dataclass
class VersionInfo:
    """存储特定版本的信息"""
    issue_id: int
    round_num: int
    requirement: str
    agent_response: str
    modified_files: List[str] = None


class VersionManager:
    """管理代码生成的版本信息，支持版本回退和需求整合"""

    def __init__(self, issue_id: int, ai_config: AIConfig, log_manager: LogManager, git_manager: GitManager, file_memory=None):
        """
        初始化版本管理器
        
        Args:
            issue_id: GitHub issue编号
            ai_config: AI配置
            log_manager: 日志管理器实例 
            git_manager: Git管理器实例
            file_memory: 文件内存管理器实例(可选)
        """
        self.ai_assistant = AIAssistant(config=ai_config, tools=[self._create_version_manager_tool()])
        self.file_memory = file_memory
        self.log_manager = log_manager
        self.git_manager = git_manager
        self.current_issue_id = issue_id
        self.current_round_num = log_manager.get_current_round()

    def ensure_version_and_generate_context(self, original_requirement: str) -> tuple[str, str]:
        requirement = None
        if self.current_round_num > 1 :
            rollback, rollback_num, requirement, reasoning = self._analyze_rollback_need(original_requirement)
        requirement = original_requirement if requirement is None else requirement
        history = self.get_formatted_history()
        return requirement, history
    
    def finalize_changes(self, mode: str, comment_text: str) -> bool:
        """
        完成更改，在bot模式下提交并推送更改
        
        Args:
            mode: 工作模式 ("client" 或 "bot")
            comment_text: comment内容

        Returns:
            bool: 操作是否成功
        """
        if mode == "bot":
            branch_name = f"bella-bot-issues-{self.current_issue_id}"
            self.git_manager.commit(f"Issues #{self.current_issue_id} - Changes by Bella-Issues-Bot")
            self.git_manager.push(branch=branch_name, force=False, set_upstream=True)
            self.git_manager.add_issue_comment(self.current_issue_id, comment_text)
        return True

    
    def _extract_history(self) -> List[VersionInfo]:
        """
        提取当前issue的历史版本信息
            
        Returns:
            List[VersionInfo]: 历史版本信息列表
        """
        # 获取所有轮次的日志条目
        log_entries = self.log_manager.get_issue_log_entries()
        
        # 提取每轮的需求和响应
        version_info_list = []
        for entry in log_entries:
            try:
                # 从用户提示中提取需求
                extracted_info = PromptGenerator.extractInfo(entry.prompt)
                requirement = extracted_info.requirement
                
                # 创建版本信息
                version_info = VersionInfo(
                    issue_id=self.current_issue_id,
                    round_num=entry.round_num,
                    requirement=requirement,
                    agent_response=entry.response,
                    modified_files=entry.modified_files
                )
                version_info_list.append(version_info)
                
            except Exception as e:
                logger.error(f"提取轮次 {entry.round_num} 的信息时出错: {str(e)}")
        
        return version_info_list

    def get_formatted_history(self) -> str:
        """
        获取格式化的历史执行记录
        
        Args:
            
        Returns:
            str: 格式化的历史执行记录
        """
        history = self._extract_history()
        formatted_history = []
        
        for version in history:
            formatted_history.append(f"【round_{version.round_num}】")
            formatted_history.append(f"requirement: \n{version.requirement}")
            
            # 简化AI响应，避免历史记录过长
            formatted_history.append(f"agent_response: \n{version.agent_response}")
            formatted_history.append("")  # 添加空行分隔
        return "\n".join(formatted_history)

    def _analyze_rollback_need(self,
                              current_requirement: str) -> Tuple[bool, Optional[int], Optional[str], Optional[str]]:
        """
        分析是否需要版本回退
        
        Args:
            current_requirement: 当前用户需求
            
        Returns:
            Tuple[bool, int, str, str]: (是否需要回退, 回退到的轮次, 整合后的需求，决策的原因)
        """
        # 获取历史记录
        history = self.get_formatted_history()
        
        # 构建提示词
        prompt = f"""
# 角色
你是一位资深程序员，现在在处理用户的issues，请分析本次用户提出的需求和历史执行记录，判断之前提交的代码是否需要版本回退。并调用工具完成版本的创建。

#注意
需要重置的轮次如果就是上一个轮次，那么就是不需要回退。

##需求回退的情况举例
- 最后一个round是5，假如你认为round4和round5的改动极其不符合，会影响用户需求的实现，那么就需要回滚4和5，此次的需求基于round3修改，target_round为1
- 最后一个round是4，假如你认为之前的所有修改都极其不符合用户需求或错误，重新改更有利，那么用户此次的需求应该基于原始代码修改，那么就需要回滚所有改动，target_round为0

##不需要回滚的情况举例
- 最后一个round是5，假如你认为用户的此次需求，在round5的代码上改动即可，那么就可以不回滚，直接修改

# 历史执行记录
{history}

# 当前用户需求
{current_requirement}


#执行步骤
##工具参数分析
你需要调用 version_manager 工具来处理问题，请请根据上述信息分析以下问题，
1. 是否需要回退到某个特定版本? 如果需要，则调用工具时的参数 need_rollback 为 True
2. 如果需要回退，应该回退到哪个round?  调用工具时的参数，大于等于0，如果全部回滚则设置为0
3. 如果需要回退，当前需求的信息是否完整？需要把回退到的round之后的round需求与当前需求结合，作为补充信息吗? 如果需要结合，则将重写后的本轮需求，作为integrated_requirement参数；如果不需要则不需要此参数。
4. 做出这个决策的原因是什么？调用工具时，作为reasoning参数

##工具执行
根据得到的参数，调用version_manager。无论是否需要rollback必须调用工具执行任务，完成当前版本的创建。

"""

        # 发送给AI进行分析
        response = self.ai_assistant.generate_response(prompt, use_tools=True)

        return response if response else (False, 0, current_requirement)

    def _rollback_to_version(self, target_round: int) -> bool:
        """
        执行基于文件的版本回退
        
        Args:
            target_round: 目标轮次
            
        Returns:
            bool: 回退是否成功
        """
        try:
            # 获取目标轮次的日志条目
            log_entries = self.log_manager.get_issue_log_entries(include_diff=True)
            
            if not log_entries:
                logger.error("找不到任何日志条目")
                return False
            
            # 获取当前轮次到目标轮次之间的所有轮次记录，按轮次降序排列（从最新到最旧）
            entries_to_rollback = sorted(
                [entry for entry in log_entries if entry.round_num > target_round],
                key=lambda entry: entry.round_num,
                reverse=True
            )
            
            if not entries_to_rollback:
                logger.info(f"当前已经是轮次 {target_round} 或更早，无需回滚")
                return True
            
            # 依次回滚每一轮的修改
            import os
            project_dir = self.log_manager.config.project_dir
            
            for entry in entries_to_rollback:
                logger.info(f"正在回滚轮次 {entry.round_num} 的修改...")
                
                if not entry.modified_files:
                    logger.info(f"轮次 {entry.round_num} 没有文件修改，跳过")
                    continue
                
                # 对每个修改的文件进行回滚
                for diff_info in entry.modified_files:
                    file_path = os.path.join(project_dir, diff_info.file_name)
                    
                    if diff_info.is_create:
                        # 如果是创建操作，则删除该文件
                        if os.path.exists(file_path):
                            os.remove(file_path)
                            logger.info(f"删除文件 {diff_info.file_name}，回滚轮次 {entry.round_num} 的创建操作")
                    
                    elif diff_info.is_modify:
                        # 如果是修改操作，则恢复到修改前的内容
                        if diff_info.file_content is not None:
                            os.makedirs(os.path.dirname(file_path), exist_ok=True)
                            with open(file_path, "w", encoding="utf-8") as f:
                                f.write(diff_info.file_content)
                            logger.info(f"恢复文件 {diff_info.file_name} 到轮次 {entry.round_num} 修改前的状态")
                        else:
                            logger.warning(f"文件 {diff_info.file_name} 没有保存修改前的内容，无法回滚")
                    
                    elif diff_info.is_delete:
                        # 如果是删除操作，则恢复文件
                        if diff_info.file_content is not None:
                            os.makedirs(os.path.dirname(file_path), exist_ok=True)
                            with open(file_path, "w", encoding="utf-8") as f:
                                f.write(diff_info.file_content)
                            logger.info(f"恢复文件 {diff_info.file_name}，回滚轮次 {entry.round_num} 的删除操作")
                        else:
                            logger.warning(f"文件 {diff_info.file_name} 没有保存删除前的内容，无法回滚")

            self.log_manager.rollback_logs(target_round)
            logger.info(f"成功回滚到轮次 {target_round}")
            return True
            
        except Exception as e:
            logger.error(f"文件版本回退失败: {str(e)}")
            return False

    def _rollback_to_version_git(self, target_round: int) -> bool:
        """
        执行版本回退
        
        Args:
            target_round: 目标轮次
            
        Returns:
            bool: 回退是否成功
        """
        try:
            # 构建目标分支名
            target_branch = f"bella-bot-issues-{self.current_issue_id}-{target_round}"
            return self.git_manager.reset_to(target_branch)
            
        except Exception as e:
            logger.error(f"Git版本回退失败: {str(e)}")
            return False

    class _VersionManagerToolSchema(BaseModel):
        need_rollback: bool = Field(
            ...,
            examples=[True, False],
            description="是否需要回退版本"
        )
        target_round: Optional[int] = Field(
            None,
            examples=[0,1,2,3,4],
            description="要回滚到的目标轮次，只有need_rollback为True时需要且必须.0代表需要全部回滚。"
        )
        integrated_requirement: Optional[str] = Field(
            None,
            description="整合后的需求，只有need_rollback为True且需要重写需求时需要"
        )
        reasoning: Optional[str] = Field(
            None,
            description="做这个出决策的原因"
        )

    def _create_version_manager_tool(self) -> StructuredTool:
        """
        版本管理工具，供AI助手使用
            
        Returns:
            Tool: 版本管理工具
        """
        def version_manager_tool(need_rollback: bool,
                                          target_round: Optional[int] = None,
                                          integrated_requirement: Optional[str] = None,
                                          reasoning: Optional[str] = None) -> Tuple[str, Optional[int], Optional[str], Optional[str]]:
            """
            决定是否回退版本并执行回退
            
            Args:
                need_rollback:是否需要回退,
                target_round：回退到的轮次,
                integrated_requirement： 整合后的需求
                reasoning: 做出决策的原因
                
            Returns:
                str: 执行结果
            """
            
            if need_rollback and target_round is not None:
                success = self._rollback_to_version(target_round)
                if(success) :
                    return (True, target_round, integrated_requirement, reasoning)
                else:
                    logger.warning(f"版本回退失败:issues:{self.current_issue_id},target:{target_round},integrated_requirement:{integrated_requirement}")
            return (False, 0, None, reasoning)
        
        return StructuredTool.from_function(
            name="version_rollback_manager",
            description="用于决定当前项目版本的工具，如果 need_rollback 为True，则根据target_round和integrated_requirement进行版本回退；如果need_rollback为False则保持当前版本",
            func=version_manager_tool,
            return_direct=True,
            args_schema=self._VersionManagerToolSchema
        )



import datetime
import os
import json
import shutil
from dataclasses import dataclass, field
from typing import List, Optional
from pydantic import BaseModel

from core.diff import DiffInfo
from core.log_config import get_logger

logger = get_logger(__name__)

@dataclass
class LogConfig:
    """日志管理配置"""

    project_dir: str
    issue_id: int
    mode: str = "client" # ["client", "bot"]


class LogEntry(BaseModel):
    """存储单次代码生成日志的数据类"""
    issue_id: int
    round_num: int
    sys_prompt: str
    prompt: str
    response: str
    timestamp: str = datetime.datetime.now().isoformat()
    log_path: str = ""
    modified_files: List[DiffInfo] = []


class LogManager:
    """管理代码生成日志的存档和检索"""
    base_dir: str = ".eng"
    logs_base_dir: str = "memory"
    rollback_dir: str = "rollback"

    def __init__(self, config: LogConfig):
        """
        初始化日志管理器

        Args:
            config: LogConfig实例，包含必要的配置信息
        """
        self.config = config
        self.issue_id = self.config.issue_id

        
        # 根据模式选择存储目录
        if self.config.mode == "bot":
            logs_dir = "issues"
        else:
            logs_dir = "queries"
            
        # 构建完整的日志存储路径
        self.logs_path = os.path.join(
            self.config.project_dir, 
            self.base_dir,
            self.logs_base_dir,
            logs_dir)
        self.issues_path = os.path.join(
            self.logs_path, "#" + str(self.issue_id)
        )

        self.rollback_path = os.path.join(
            self.logs_path, "#" + str(self.issue_id), self.rollback_dir
        )

        # 初始化当前轮次
        self.current_round = self._get_next_round()

        # 确保必要的目录存在
        os.makedirs(self.issues_path, exist_ok=True)
        os.makedirs(self.rollback_path, exist_ok=True)

        # 定义日志文件名常量
        self.SYS_PROMPT_FILE = "system_prompt.txt"
        self.USER_PROMPT_FILE = "user_prompt.txt"
        self.AI_RESPONSE_FILE = "ai_response.txt"
        self.TIMESTAMP_FILE = "timestamp.txt"
        self.MODIFIED_FILES_FILE = "modified_files.txt"

    def archive_logs(self, sys_prompt: str, prompt: str, response: str, diff_infos: List[DiffInfo] = None) -> str:
        """
        将代码生成日志存档到指定的目录

        Args:
            diff_infos: 文件的修改信息
            sys_prompt: 系统提示词
            prompt: 用户提示词
            response: AI响应

        Returns:
            str: 存档目录的路径
        """
        # 获取下一个轮次号

        round_num = self.current_round

        # 获取当前时间戳
        timestamp = datetime.datetime.now().isoformat()
        
        # 创建轮次目录
        round_dir = os.path.join(self.issues_path, f"round_{round_num}")
        os.makedirs(round_dir, exist_ok=True)
        
        # 保存系统提示词
        with open(os.path.join(round_dir, self.SYS_PROMPT_FILE), "w", encoding="utf-8") as f:
            f.write(sys_prompt)
        
        # 保存用户提示词
        with open(os.path.join(round_dir, self.USER_PROMPT_FILE), "w", encoding="utf-8") as f:
            f.write(prompt)
        
        # 保存AI响应
        with open(os.path.join(round_dir, self.AI_RESPONSE_FILE), "w", encoding="utf-8") as f:
            f.write(response)
            
        # 保存时间戳
        with open(os.path.join(round_dir, self.TIMESTAMP_FILE), "w", encoding="utf-8") as f:
            f.write(timestamp)

        # 保存修改的文件列表
        if diff_infos:
            # 使用 Pydantic 的 dict 方法进行序列化
            diff_dicts = [diff.dict() for diff in diff_infos]
                
            # 序列化为 JSON 并保存
            with open(os.path.join(round_dir, self.MODIFIED_FILES_FILE), "w", encoding="utf-8") as f:
                json.dump(diff_dicts, f, ensure_ascii=False, indent=2)
            logger.info(f"保存了 {len(diff_infos)} 个修改的文件记录")
        
        # 记录日志
        logger.info(f"已将日志存档至: {round_dir}")
        
        # 返回存档目录的路径
        return round_dir

    def _get_next_round(self) -> int:
        """
        获取下一个轮次号

        Returns:
            int: 下一个轮次号
        """
        issue_path = self.issues_path
        if not os.path.exists(issue_path):
            return 1

        existing_rounds = [
            int(d[6:])
            for d in os.listdir(issue_path)
            if os.path.isdir(os.path.join(issue_path, d)) and (d[6:]).isdigit()
        ]

        return max(existing_rounds, default=0) + 1

    def get_current_round(self) -> int:
        return self.current_round
        
    def get_issue_log_entries(self, include_diff: bool = False) -> List[LogEntry]:
        """
        获取当前issue的所有轮次的日志条目

        Returns:
            List[LogEntry]: 日志条目列表，按轮次排序
        """
        
        log_entries = []

        issue_dir = self.issues_path
        # 遍历所有轮次目录
        for dir_name in os.listdir(issue_dir):
            if not dir_name.startswith("round_"):
                continue
                
            try:
                round_num = int(dir_name[6:])  # 提取轮次号
                round_dir = os.path.join(issue_dir, dir_name)
                
                # 读取系统提示词
                sys_prompt_path = os.path.join(round_dir, self.SYS_PROMPT_FILE)
                with open(sys_prompt_path, "r", encoding="utf-8") as f:
                    sys_prompt = f.read()
                
                # 读取用户提示词
                user_prompt_path = os.path.join(round_dir, self.USER_PROMPT_FILE)
                with open(user_prompt_path, "r", encoding="utf-8") as f:
                    prompt = f.read()
                
                # 读取AI响应
                ai_response_path = os.path.join(round_dir, self.AI_RESPONSE_FILE)
                with open(ai_response_path, "r", encoding="utf-8") as f:
                    response = f.read()
                    
                # 读取修改的文件列表(如果存在)
                modified_files = []
                if include_diff:
                    modified_files_path = os.path.join(round_dir, self.MODIFIED_FILES_FILE)
                    if os.path.exists(modified_files_path):
                        try:
                            with open(modified_files_path, "r", encoding="utf-8") as f:
                                diff_dicts = json.load(f)
                                
                                # 将字典转换回 DiffInfo 对象
                                modified_files = [DiffInfo(**diff_dict) for diff_dict in diff_dicts]
                        except Exception as e:
                            logger.error(f"读取修改文件列表失败: {str(e)}")
                    
                # 读取时间戳
                timestamp_path = os.path.join(round_dir, self.TIMESTAMP_FILE)
                timestamp = datetime.datetime.now().isoformat()  # 默认当前时间
                if os.path.exists(timestamp_path):
                    try:
                        with open(timestamp_path, "r", encoding="utf-8") as f:
                            timestamp = f.read().strip()
                    except Exception as e:
                        logger.error(f"读取时间戳失败: {str(e)}")
                
                # 创建并添加LogEntry对象
                entry = LogEntry(issue_id=self.issue_id, round_num=round_num,
                                 sys_prompt=sys_prompt, prompt=prompt, 
                                 response=response, timestamp=timestamp, log_path=round_dir,
                                 modified_files=modified_files)
                log_entries.append(entry)
            except Exception as e:
                logger.error(f"读取轮次 {dir_name} 的日志失败: {str(e)}")
        
        # 按轮次号排序
        return sorted(log_entries, key=lambda entry: entry.round_num)
    
    def get_issue_round_log_entry(self, round_num: int, include_diff: bool = False) -> Optional[LogEntry]:
        """
        获取特定轮次的日志条目

        Args:
            round_num: 轮次号
            include_diff: 是否包含修改信息

        Returns:
            Optional[LogEntry]: 指定轮次的日志条目，如果不存在则返回None
        """
        round_dir = os.path.join(self.issues_path, f"round_{round_num}")
        
        if not os.path.exists(round_dir):
            logger.warning(f"Issue #{self.issue_id} 的轮次 {round_num} 不存在")
            return None
            
        try:
            # 直接使用现有方法获取所有轮次，然后过滤出指定轮次
            all_entries = self.get_issue_log_entries(include_diff)
            return next((entry for entry in all_entries if entry.round_num == round_num), None)
        except Exception as e:
            logger.error(f"获取 Issue #{self.issue_id} 轮次 {round_num} 的日志失败: {str(e)}")
            return None
            
    def rollback_logs(self, target_round: int) -> bool:
        """
        将目标轮次之后的日志移至回滚目录
        
        Args:
            target_round: 保留到的轮次，之后的轮次会被移到回滚目录
            
        Returns:
            bool: 操作是否成功
        """
        try:
            # 确保回滚目录存在
            os.makedirs(self.rollback_path, exist_ok=True)
            
            # 获取所有轮次目录
            round_dirs = [d for d in os.listdir(self.issues_path) 
                         if os.path.isdir(os.path.join(self.issues_path, d)) 
                         and d.startswith("round_")]
            
            # 筛选出需要回滚的轮次目录
            rounds_to_rollback = []
            for dir_name in round_dirs:
                try:
                    round_num = int(dir_name[6:])  # 提取轮次号
                    if round_num > target_round:
                        rounds_to_rollback.append((round_num, dir_name))
                except ValueError:
                    continue
            
            # 按轮次号排序
            rounds_to_rollback.sort(key=lambda x: x[0])
            
            if not rounds_to_rollback:
                logger.info(f"没有轮次需要回滚")
                return True
                
            # 移动轮次日志到回滚目录
            for round_num, dir_name in rounds_to_rollback:
                source_path = os.path.join(self.issues_path, dir_name)
                dest_path = os.path.join(self.rollback_path, dir_name)
                
                # 如果目标路径已存在，先删除
                if os.path.exists(dest_path):
                    shutil.rmtree(dest_path)
                    
                # 移动目录
                shutil.move(source_path, dest_path)
                logger.info(f"已将轮次 {round_num} 的日志移至回滚目录: {dest_path}")
            
            # 更新当前轮次
            self.current_round = self._get_next_round()
            return True
            
        except Exception as e:
            logger.error(f"回滚日志失败: {str(e)}")
            return False
    
    def get_rollback_log_entries(self, include_diff: bool = False) -> List[LogEntry]:
        """
        获取已回滚的所有轮次的日志条目

        Args:
            include_diff: 是否包含diff信息

        Returns:
            List[LogEntry]: 回滚的日志条目列表，按轮次排序
        """
        try:
            # 如果回滚目录不存在，返回空列表
            if not os.path.exists(self.rollback_path):
                return []
                
            # 临时保存当前issues_path
            original_path = self.issues_path
            
            # 将issues_path指向rollback_path，复用get_issue_log_entries方法
            self.issues_path = self.rollback_path
            
            # 获取回滚目录中的日志条目
            rollback_entries = self.get_issue_log_entries(include_diff)
            
            # 恢复issues_path
            self.issues_path = original_path
            
            return rollback_entries
        except Exception as e:
            logger.error(f"获取回滚日志条目失败: {str(e)}")
            return []

if __name__ == "__main__":
    config = LogConfig("..", 1)
    log_manager = LogManager(config)
